@@ -5,11 +5,12 @@ use Mojo::IOLoop;
 use Mojo::IOLoop::Client;
 use Mojo::IOLoop::Delay;
 use Mojo::IOLoop::Stream;
+use Mojo::SMTP::Client::Response;
 use Mojo::SMTP::Client::Exception;
 use Carp;
 use Scalar::Util 'weaken';
 
-our $VERSION = 0.05;
+our $VERSION = 0.06;
 
 use constant {
 	CMD_OK       => 2,
@@ -24,8 +25,6 @@ use constant {
 	CMD_DATA_END => 7,
 	CMD_RESET    => 8,
 	CMD_QUIT     => 9,
-	
-	CRLF         => "\x0d\x0a",
 };
 
 our %CMD = (
@@ -64,9 +63,9 @@ sub send {
 		my ($delay, $resp) = @_;
 		$this->emit(response => $this->{last_cmd}, $resp);
 		
-		die $resp->{error} if $resp->{error};
-		substr($resp->{code}, 0, 1) == $expected_code
-			or Mojo::SMTP::Client::Exception::Response->throw($resp->{code}, $resp->{code}.' '.join("\n", @{$resp->{messages}}));
+		unless (substr($resp->code, 0, 1) == $expected_code) {
+			die $resp->error(Mojo::SMTP::Client::Exception::Response->new($resp->message)->code($resp->code));
+		}
 		$delay->pass($resp);
 	};
 	
@@ -106,7 +105,7 @@ sub send {
 				delete($this->{cleanup_cb})->() if $this->{cleanup_cb};
 				$this->{delay}->remaining([]); # remove remaining steps
 				$this->_rm_stream();
-				$this->{delay}->emit(finish => {error => $_[0]});
+				$this->{delay}->emit(finish => Mojo::SMTP::Client::Response->new('')->error($_[0]));
 			};
 			
 			$this->{stream} = Mojo::IOLoop::Stream->new($_[0]);
@@ -135,8 +134,8 @@ sub send {
 		}, 
 		sub {
 			eval { $resp_checker->(@_); $_[1]->{checked} = 1 };
-			if (my $err = $@) {
-				die $err unless $err->isa('Mojo::SMTP::Client::Exception::Response');
+			if (my $e = $@) {
+				die $e unless $e->error->isa('Mojo::SMTP::Client::Exception::Response');
 				my $delay = shift;
 				$this->_cmd('HELO ' . $this->hello, CMD_HELO);
 				$this->_read_response($delay->begin, 0);
@@ -205,7 +204,7 @@ sub send {
 					my $data = $cmd[$mi]->();
 					
 					unless (length(ref $data ? $$data : $data) > 0) {
-						$this->_cmd(($was_nl ? '' : CRLF).'.', CMD_DATA_END);
+						$this->_cmd(($was_nl ? '' : Mojo::SMTP::Client::Response::CRLF).'.', CMD_DATA_END);
 						$this->_read_response($data_writer_cb, $mi == $#cmd);
 						$expected_code = CMD_OK;
 						return delete($this->{cleanup_cb})->();
@@ -224,7 +223,7 @@ sub send {
 				},
 				sub {
 					my $delay = shift;
-					$this->_cmd((_has_nl($cmd[$mi]) ? '' : CRLF).'.', CMD_DATA_END);
+					$this->_cmd((_has_nl($cmd[$mi]) ? '' : Mojo::SMTP::Client::Response::CRLF).'.', CMD_DATA_END);
 					$this->_read_response($delay->begin, $mi == $#cmd);
 					$expected_code = CMD_OK;
 				},
@@ -260,7 +259,7 @@ sub send {
 	
 	# non-blocking
 	my $delay = $this->{delay} = Mojo::IOLoop::Delay->new(ioloop => $this->_ioloop($nb))->steps(@steps)->catch(sub {
-		shift->emit(finish => {error => $_[0]});
+		shift->emit(finish => $_[0]);
 	});
 	$delay->on(finish => sub {
 		if ($cb) {
@@ -277,7 +276,7 @@ sub send {
 			$resp = pop;
 		};
 		$delay->wait;
-		return $self->autodie && $resp->{error} ? die $resp->{error} : $resp;
+		return $self->autodie && $resp->error ? die $resp->error : $resp;
 	}
 }
 
@@ -294,7 +293,7 @@ sub _server {
 sub _cmd {
 	my ($self, $cmd, $cmd_const) = @_;
 	$self->{last_cmd} = $cmd_const;
-	$self->{stream}->write($cmd.CRLF);
+	$self->{stream}->write($cmd.Mojo::SMTP::Client::Response::CRLF);
 }
 
 sub _rm_stream {
@@ -319,26 +318,11 @@ sub _read_response {
 				$self->{stream}->timeout(0);
 				$self->{stream}->stop;
 			}
-			$cb->($self, _parse_response($resp));
+			$cb->($self, Mojo::SMTP::Client::Response->new($resp));
 		}
 	});
 }
 
-sub _parse_response {
-	my ($code, @msg);
-	
-	my @lines = split CRLF, $_[0];
-	($code) = $lines[0] =~ /^(\d+)/;
-	
-	for (@lines) {
-		if (/^\d+[-\s](.+)/) {
-			push @msg, $1;
-		}
-	}
-	
-	return {code => $code, messages => \@msg};
-}
-
 sub _has_nl {
 	if (ref $_[0]) {
 		return ${$_[0]} =~ /\012$/;
@@ -402,7 +386,7 @@ Mojo::SMTP::Client - non-blocking SMTP client based on Mojo::IOLoop
 		quit => 1,
 		sub {
 			my ($smtp, $resp) = @_;
-			warn $resp->{error} ? 'Failed to send: '.$resp->{error} : 'Sent successfully';
+			warn $resp->error ? 'Failed to send: '.$resp->error : 'Sent successfully';
 			Mojo::IOLoop->stop;
 		}
 	);
@@ -441,7 +425,7 @@ Emitted whenever a new connection is about to start.
 	});
 
 Emitted for each SMTP response from the server. C<$cmd> is a command L<constant|/CONSTANTS> for which this
-response was sent. For C<$resp> description see L</send>.
+response was sent. C<$resp> is L<Mojo::SMTP::Client::Response> object.
 
 =head1 ATTRIBUTES
 
@@ -481,7 +465,7 @@ Event loop object to use for blocking I/O operations, defaults to a L<Mojo::IOLo
 Defines should or not C<Mojo::SMTP::Client> throw exceptions for any type of errors. This only usable for
 blocking usage of C<Mojo::SMTP::Client>, because non-blocking one should never die. Throwed
 exception will be one of the specified in L<Mojo::SMTP::Client::Exception>. When autodie attribute
-has false value you should check C<$respE<gt>{error}> yourself. Default is false.
+has false value you should check C<$respE<gt>error> yourself. Default is false.
 
 =head1 METHODS
 
@@ -541,12 +525,10 @@ the connection until last email will be sent.
 =back
 
 For non-blocking usage last argument to C<send> should be reference to subroutine which will be called when result will
-be available. Subroutine arguments will be C<($smtp, $resp)>. Where C<$resp> is reference to a hash with
-response. This hash may has this keys: C<error>, C<code>, C<messages>. First you should check C<error> - 
-if it has true value this means that it was error somewhere while sending. C<$resp-E<gt>{error}> will be one
-of C<Mojo::SMTP::Client::Exception::*> objects defined in L<Mojo::SMTP::Client::Exception>. If C<error> has
-false value you can get code and messages for last command with C<$resp-E<gt>{code}> (number) and
-C<$resp-E<gt>{messages}> (reference to array with strings).
+be available. Subroutine arguments will be C<($smtp, $resp)>. Where C<$resp> is object of L<Mojo::SMTP::Client::Response> class.
+First you should check C<$resp-E<gt>error> - if it has true value this means that it was error somewhere while sending.
+If C<error> has false value you can get code and message for response to last command with C<$resp-E<gt>code> (number) and
+C<$resp-E<gt>message> (string).
 
 For blocking usage C<$resp> will be returned as result of C<$smtp-E<gt>send> call. C<$resp> is the same as for
 non-blocking result. If L</autodie> attribute has true value C<send> will throw an exception on any error.
@@ -750,7 +732,7 @@ of necessary SMTP server. We'll get it with help of L<Net::DNS>. Then we'll send
 			my ($smtp, $cmd, $resp) = @_;
 			
 			print ">>", $Mojo::SMTP::Client::CMD{$cmd}, "\n";
-			print "<<", $resp->{code}, " ", join("\n", @{$resp->{messages}}), "\n";
+			print "<<", $resp, "\n";
 		});
 		
 		$smtp->send(
@@ -761,8 +743,8 @@ of necessary SMTP server. We'll get it with help of L<Net::DNS>. Then we'll send
 			sub {
 				my ($smtp, $resp) = @_;
 				
-				warn $resp->{error} ? 'Failed to send: '.$resp->{error} :
-				                      'Sent successfully with code: ', $resp->{code};
+				warn $resp->error ? 'Failed to send: '.$resp->error :
+				                      'Sent successfully with code: ', $resp->code;
 				
 				$loop->stop;
 			}
@@ -777,7 +759,7 @@ on your domain and so on.
 
 =head1 SEE ALSO
 
-L<Mojo::SMTP::Client::Exception>, L<Mojolicious>, L<Mojo::IOLoop>
+L<Mojo::SMTP::Client::Response>, L<Mojo::SMTP::Client::Exception>, L<Mojolicious>, L<Mojo::IOLoop>
 
 =head1 COPYRIGHT
 
